import React, { useState, useEffect, useRef } from 'react';
import { Button, Card, Row, Col, Space, Typography, message } from 'antd';
import { 
  PhoneOutlined, 
  VideoCameraOutlined, 
  AudioOutlined, 
  AudioMutedOutlined,
  VideoCameraAddOutlined,
  StopOutlined 
} from '@ant-design/icons';
import { Room, RemoteParticipant, LocalParticipant } from 'twilio-video';
import { twilioService, VideoCallOptions, ParticipantInfo } from '../../services/twilioService';

const { Title, Text } = Typography;

interface VideoCallProps {
  roomName: string;
  identity: string;
  onLeave?: () => void;
}

interface ParticipantVideoProps {
  participant: RemoteParticipant | LocalParticipant;
  isLocal: boolean;
}

const ParticipantVideo: React.FC<ParticipantVideoProps> = ({ participant, isLocal }) => {
  const videoRef = useRef<HTMLVideoElement>(null);
  const audioRef = useRef<HTMLAudioElement>(null);

  useEffect(() => {
    let videoTrack: any = null;
    let audioTrack: any = null;
    
    // 获取第一个视频轨道
    participant.videoTracks.forEach((track) => {
      if (!videoTrack) videoTrack = track;
    });
    
    // 获取第一个音频轨道
    participant.audioTracks.forEach((track) => {
      if (!audioTrack) audioTrack = track;
    });

    if (videoTrack && videoRef.current) {
      videoTrack.track?.attach(videoRef.current);
    }

    if (audioTrack && audioRef.current && !isLocal) {
      audioTrack.track?.attach(audioRef.current);
    }

    return () => {
      if (videoTrack?.track) {
        videoTrack.track.detach();
      }
      if (audioTrack?.track && !isLocal) {
        audioTrack.track.detach();
      }
    };
  }, [participant, isLocal]);

  return (
    <div className="participant-video" style={{ position: 'relative', width: '100%', height: '200px' }}>
      <video
        ref={videoRef}
        autoPlay
        muted={isLocal}
        style={{
          width: '100%',
          height: '100%',
          objectFit: 'cover',
          borderRadius: '8px',
        }}
      />
      {!isLocal && <audio ref={audioRef} autoPlay />}
      <div
        style={{
          position: 'absolute',
          bottom: '8px',
          left: '8px',
          background: 'rgba(0,0,0,0.6)',
          color: 'white',
          padding: '4px 8px',
          borderRadius: '4px',
          fontSize: '12px',
        }}
      >
        {participant.identity} {isLocal && '(您)'}
      </div>
    </div>
  );
};

export const VideoCall: React.FC<VideoCallProps> = ({ roomName, identity, onLeave }) => {
  const [room, setRoom] = useState<Room | null>(null);
  const [participants, setParticipants] = useState<ParticipantInfo[]>([]);
  const [isConnecting, setIsConnecting] = useState(false);
  const [isConnected, setIsConnected] = useState(false);
  const [audioEnabled, setAudioEnabled] = useState(true);
  const [videoEnabled, setVideoEnabled] = useState(true);

  const connectToRoom = async () => {
    if (isConnecting) return;

    setIsConnecting(true);
    try {
      const options: VideoCallOptions = {
        roomName,
        identity,
        audio: audioEnabled,
        video: videoEnabled,
      };

      const connectedRoom = await twilioService.connectToRoom(options);
      setRoom(connectedRoom);
      setIsConnected(true);
      
      setupRoomEventListeners(connectedRoom);
      
      message.success('成功连接到视频通话');
    } catch (error) {
      console.error('连接失败:', error);
      message.error('连接视频通话失败');
    } finally {
      setIsConnecting(false);
    }
  };

  const setupRoomEventListeners = (room: Room) => {
    const updateParticipants = () => {
      setParticipants(twilioService.getParticipants());
    };

    room.on('participantConnected', (participant: RemoteParticipant) => {
      message.info(`${participant.identity} 加入了通话`);
      updateParticipants();
    });

    room.on('participantDisconnected', (participant: RemoteParticipant) => {
      message.info(`${participant.identity} 离开了通话`);
      updateParticipants();
    });

    room.on('disconnected', () => {
      setIsConnected(false);
      setRoom(null);
      setParticipants([]);
      message.info('已断开视频通话连接');
    });

    updateParticipants();
  };

  const leaveRoom = () => {
    twilioService.disconnect();
    setIsConnected(false);
    setRoom(null);
    setParticipants([]);
    onLeave?.();
  };

  const toggleAudio = () => {
    const newAudioState = twilioService.toggleAudio();
    setAudioEnabled(newAudioState);
  };

  const toggleVideo = () => {
    const newVideoState = twilioService.toggleVideo();
    setVideoEnabled(newVideoState);
  };

  useEffect(() => {
    return () => {
      twilioService.disconnect();
    };
  }, []);

  if (!isConnected) {
    return (
      <Card style={{ width: '100%', maxWidth: '400px', margin: '0 auto' }}>
        <div style={{ textAlign: 'center' }}>
          <Title level={4}>视频通话</Title>
          <Text>房间: {roomName}</Text>
          <br />
          <Text>身份: {identity}</Text>
          <br /><br />
          
          <Space direction="vertical" style={{ width: '100%' }}>
            <Row gutter={16}>
              <Col span={12}>
                <Button
                  type={audioEnabled ? 'primary' : 'default'}
                  icon={audioEnabled ? <AudioOutlined /> : <AudioMutedOutlined />}
                  onClick={() => setAudioEnabled(!audioEnabled)}
                  style={{ width: '100%' }}
                >
                  {audioEnabled ? '音频开启' : '音频关闭'}
                </Button>
              </Col>
              <Col span={12}>
                <Button
                  type={videoEnabled ? 'primary' : 'default'}
                  icon={<VideoCameraOutlined />}
                  onClick={() => setVideoEnabled(!videoEnabled)}
                  style={{ width: '100%' }}
                >
                  {videoEnabled ? '视频开启' : '视频关闭'}
                </Button>
              </Col>
            </Row>
            
            <Button
              type="primary"
              size="large"
              icon={<VideoCameraAddOutlined />}
              loading={isConnecting}
              onClick={connectToRoom}
              style={{ width: '100%' }}
            >
              {isConnecting ? '连接中...' : '加入通话'}
            </Button>
          </Space>
        </div>
      </Card>
    );
  }

  return (
    <div style={{ width: '100%', height: '100vh', padding: '16px' }}>
      <Card style={{ height: '100%' }}>
        <div style={{ display: 'flex', flexDirection: 'column', height: '100%' }}>
          <div style={{ marginBottom: '16px', textAlign: 'center' }}>
            <Title level={4}>视频通话 - {roomName}</Title>
            <Text>参与者: {participants.length}</Text>
          </div>

          <div style={{ flex: 1, overflow: 'auto' }}>
            <Row gutter={[16, 16]}>
              {participants.map((participantInfo) => {
                const participant = participantInfo.isLocal 
                  ? room?.localParticipant 
                  : Array.from(room?.participants.values() || []).find(p => p.sid === participantInfo.sid);
                
                if (!participant) return null;

                return (
                  <Col xs={24} sm={12} md={8} key={participantInfo.sid}>
                    <ParticipantVideo
                      participant={participant}
                      isLocal={participantInfo.isLocal}
                    />
                  </Col>
                );
              })}
            </Row>
          </div>

          <div style={{ marginTop: '16px' }}>
            <Row gutter={16} justify="center">
              <Col>
                <Button
                  type={audioEnabled ? 'primary' : 'default'}
                  shape="circle"
                  size="large"
                  icon={audioEnabled ? <AudioOutlined /> : <AudioMutedOutlined />}
                  onClick={toggleAudio}
                />
              </Col>
              <Col>
                <Button
                  type={videoEnabled ? 'primary' : 'default'}
                  shape="circle"
                  size="large"
                  icon={<VideoCameraOutlined />}
                  onClick={toggleVideo}
                />
              </Col>
              <Col>
                <Button
                  type="primary"
                  danger
                  shape="circle"
                  size="large"
                  icon={<StopOutlined />}
                  onClick={leaveRoom}
                />
              </Col>
            </Row>
          </div>
        </div>
      </Card>
    </div>
  );
}; 